package com.example.mdd.service;

import cn.hutool.db.Db;
import cn.hutool.db.DbUtil;
import cn.hutool.db.Entity;
import cn.hutool.db.ds.simple.SimpleDataSource;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.example.mdd.entity.PowerStation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MeteorologicalDataDownload {
    @Resource
    private HeFengDownload heFengDownload;
    @Resource
    private CaiYunDownload caiYunDownload;
    @Resource
    private XinZhiDownload xinZhiDownload;
    @Resource
    private BaiDuDownload baiDuDownload;

    public void download() {
        String body = HttpUtil.get("http://itil.jiayuepowertech.com:9958/itil/api/power-station?paramStationState=01,10,20", 10000);
        JSONObject json = JSONUtil.parseObj(body);
        String code = json.get("code").toString();
        String data = json.get("data").toString();
        if (code.equals("0") && !data.isEmpty()) {
            JSONArray array = JSONUtil.parseArray(data);
            //itil所有的场站数据  根据场站数据的经纬度通过气象平台的api接口查询数据
            List<PowerStation> list = array.toList(PowerStation.class);
            String[] codes = {"J00615", "J00629", "J00525", "J00526", "J00577"};
            try {
                DataSource ds = new SimpleDataSource("jdbc:mysql://192.168.12.10:23306/mdd"
                        , "root", "la!yibei82nianxueB");
                Connection conn = DbUtil.use(ds).getConnection();
                for (PowerStation powerStation : list) {
                    if (!"".equals(powerStation.getLongitude()) && !"".equals(powerStation.getLatitude())) {
                        heFengDownload.download(conn, powerStation.getStationCode(), powerStation.getLongitude(), powerStation.getLatitude());
                        caiYunDownload.download(conn, powerStation.getStationCode(), powerStation.getLongitude(), powerStation.getLatitude());
                        xinZhiDownload.download(conn, powerStation.getStationCode(), powerStation.getLongitude(), powerStation.getLatitude());
                        if (Arrays.asList(codes).contains(powerStation.getStationCode())) {
                            baiDuDownload.download(conn, powerStation.getStationCode(), powerStation.getLongitude(), powerStation.getLatitude());
                        }
                    }
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }

        }
    }

    public void downloadHour() {
        String body = HttpUtil.get("http://itil.jiayuepowertech.com:9958/itil/api/power-station?paramStationState=01,10,20", 10000);
        JSONObject json = JSONUtil.parseObj(body);
        String code = json.get("code").toString();
        String data = json.get("data").toString();
        if (code.equals("0") && !data.isEmpty()) {
            JSONArray array = JSONUtil.parseArray(data);
            //itil所有的场站数据  根据场站数据的经纬度通过气象平台的api接口查询数据
            List<PowerStation> list = array.toList(PowerStation.class);
            try {
                DataSource ds = new SimpleDataSource("jdbc:mysql://192.168.12.10:23306/mdd"
                        , "root", "la!yibei82nianxueB");
                Connection conn = DbUtil.use(ds).getConnection();
                List<Entity> saveList = Db.use().findAll("save_station");
                log.info("开始下载彩云天气小时数据,需要下载的场站数量为" + saveList.size());
                for (Entity entity : saveList) {
                    String stationCode = entity.getStr("station_code");
                    List<PowerStation> collect = list.stream().filter(item -> item.getStationCode().equals(stationCode)).collect(Collectors.toList());
                    if (!collect.isEmpty()) {
                        if (!"".equals(collect.get(0).getLongitude()) && !"".equals(collect.get(0).getLatitude())) {
                            caiYunDownload.downloadHour(conn, collect.get(0).getStationCode(), collect.get(0).getLongitude(), collect.get(0).getLatitude());
                        }
                    }
                }
                log.info("彩云天气小时数据下载完成");
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }

        }
    }
}
